/* taglib_config.h. Generated by cmake from taglib_config.h.cmake */

#ifndef TAGLIB_TAGLIB_CONFIG_H
#define TAGLIB_TAGLIB_CONFIG_H

#define TAGLIB_WITH_APE 1
#define TAGLIB_WITH_ASF 1
#define TAGLIB_WITH_DSF 1
#define TAGLIB_WITH_MATROSKA 1
#define TAGLIB_WITH_MOD 1
#define TAGLIB_WITH_MP4 1
#define TAGLIB_WITH_RIFF 1
#define TAGLIB_WITH_SHORTEN 1
#define TAGLIB_WITH_TRUEAUDIO 1
#define TAGLIB_WITH_VORBIS 1

#endif
